%%% Matlab code to calculate the magnetic fields from a pair of permanent
%%% magnets, as they are commonly used for single-molecule magnetic tweezers
%%%
%%% This code implements the (semi-)analytical calculations based on the
%%% Biot-Savart law described in 
%%% "Quantitative modeling and optimization of magnetic tweezers"
%%% Jan Lipfert, Xiaomin Hao, and Nynke H. Dekker
%%% Biophys J. 2009 Jun 17;96(12):5040-9. doi: 10.1016/j.bpj.2009.03.055.
%%% 
%%% For notations, schematic drawings of the geometry, and examples see 
%%% the paper by Lipfert et al. Biophys J. (2009), in particular also the
%%% supplementary information. 
%%%
%%% There are two main sections of the code, to compute the fields for
%%% horizontally and vertically oriented magnets, respectively. 
%%% All calculations assume that the problems is effectively 1-D, i.e. compute
%%% the fields along the axis of symmetry along z. The point z = 0 is
%%% chosen to by at the center of symmetry between the magnets.
%%%
%%% This part of the code computes the magnetic fields, only. To compute
%%% the forces that a bead would experience in the magnetic tweezers:
%%% 1) compute the fields for the geometry of interest using this code and
%%% output the result to a file, as implemented below
%%% 2) run the companion code "Lipfert_etal_ForceCalculations.m" to compute
%%% the forces
%%%
%%% If you find this code useful, please cite it as indicated above.
%%%
%%% Author: Jan Lipfert
%%%

clear;clc;clf;
hold on; box on;

%%% Set some variables for plotting
LW = 1.5; % Linewidth
MS = 6;   % Mark size


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%   ----- Integrals for the horizontal magnet configuration      --- %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if 0
   
    %%% Set the properties of the magnets
    L = 5;   % Magnet side length in mm, assuming cubic magnets L x L x L mm
    g = 0.5; % Half the gap size in mm, i.e. half the distance between the magnets
    Br = 1.24;  %%% Remanent magnetization (field) in T
    
    syms x y z zint;
    
    %%% --- Integrate L1 and R1 --- %%%
    %%% Integrate y from -L/2 to L/2
    part1 = int( 2*(z+L/2)./(x.^2 + y.^2 + (z+L/2).^2).^(1.5), y);
    part1 = subs(part1, y, L/2) - subs(part1, y, -L/2);
    %%% Integrate x from g to g+L
    part1 = int( part1, x );
    part1 = subs(part1, x, g+L) - subs(part1, x, g);
    
    
    %%% --- Integrate L3 and R3 --- %%%
    %%% Same integral limits as above
    %%% Integrate y from -L/2 to L/2
    part2 = int( 2*(-z+L/2)./(x.^2 + y.^2 + (-z+L/2).^2).^(1.5), y);
    part2 = subs(part2, y, L/2) - subs(part2, y, -L/2);
    %%% Integrate x from g to g+L
    part2 = int( part2, x );
    part2 = subs(part2, x, g+L) - subs(part2, x, g);
    
    
    %%% --- Integrate L2, L4 and R2, R4 --- %%%
    %%% Integrate z from -L/2 to L/2
    part3 = int( 2*L./(x.^2 + (L/2).^2 + (z-zint).^2).^(1.5), zint);
    part3 = subs(part3, zint, L/2) - subs(part3, zint, -L/2);
    
    %%% Integrate x from g to g+L
    part3 = int( part3, x );
    part3 = subs(part3, x, g+L) - subs(part3, x, g);
    
    total = part1 + part2 + part3;
    
    %%% Evaluate the field at positions "zplot"  and show the results   
    zplot = 0:0.1:20;
    Bplot = subs(total, z, zplot);
    Bplot = Br/4/pi .* Bplot;
    plot(zplot, Bplot, 'linewidth', LW)
    
    ylabel('Field (T)', 'fontsize', 16, 'fontweight', 'bold');
    xlabel('Magnet distance (mm)', 'fontsize', 16, 'fontweight', 'bold');
    set(gca, 'fontsize', 16, 'linewidth', 1, 'fontweight', 'bold', 'TickLength',[0.03 0.03])

    %%% Save the results to a text file
    foo =[zplot' real(Bplot)'];
    save -ascii B_hor_5_5_5_1.txt foo

end




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%   ----- Integrals for the vertical magnet configuration  --- %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if 0
  
    %%% Set the properties of the magnets
    L = 5;   % Magnet side length in mm, assuming cubic magnets L x L x L mm
    g = 0.5; % Half the gap size in mm, i.e. half the distance between the magnets
    Br = 1.24;  %%% Remanent magnetization (field) in T
    
    syms x y z zint;
    
    %%% --- Integrate L2 and R4 --- %%%
    %%% Integrate y from -L/2 to L/2
    part1 = int( 2*(z-zint)./(g.^2 + y.^2 + (z-zint).^2).^(1.5), y);
    part1 = subs(part1, y, L/2) - subs(part1, y, -L/2);
    %%% Integrate z from -L/2 to L/2
    part1 = int( part1, zint );
    part1 = subs(part1, zint, L/2) - subs(part1, zint, -L/2);
    
    
    %%% --- Integrate L4 and R2 --- %%%
    %%% Same integral limits as above
    %%% Integrate y from -L/2 to L/2
    part2 = int( 2*(z-zint)./((g+L).^2 + y.^2 + (z-zint).^2).^(1.5), y);
    part2 = subs(part2, y, L/2) - subs(part2, y, -L/2);
    %%% Integrate z from -L/2 to L/2
    part2 = int( part2, zint );
    part2 = subs(part2, zint, L/2) - subs(part2, zint, -L/2);
   
    total = part1 - part2;
    
    %%% Evaluate the field at positions "zplot"  and show the results   
    zplot = 0:0.1:20;
    Bplot = subs(total, z, zplot);
    Bplot = Br/4/pi .* Bplot;
    plot(zplot, Bplot, 'linewidth', LW)
    
    ylabel('Field (T)', 'fontsize', 16, 'fontweight', 'bold');
    xlabel('Magnet distance (mm)', 'fontsize', 16, 'fontweight', 'bold');
    set(gca, 'fontsize', 16, 'linewidth', 1, 'fontweight', 'bold', 'TickLength',[0.03 0.03])

    %%% Save the results to a text file
    foo =[zplot' real(Bplot)'];
    save -ascii B_vert_5_5_5_1.txt foo
   
end
